import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo);

/**
 * <p>Repraesentation eines virtuellen Rubiks-Cube.
 * Direkt nach dem Konstruktor-Aufruf ist der Wuerfel vollstaendig geloest.
 * Ueber statische Attribute der Klasse ist eine Orientierung "OBEN, UNTEN,..."
 * und eine Farbnennung "ROT, BLAU, ..." festgelegt.</p>
 * 
 * <p>Die Sichtperspektive auf den Wuerfel ist dabei immer von VORNE.
 * Bei der getColor-Methode ist eine Angabe in Zeilen und Spalten unklar, wenn die
 * Art der Perspektive nicht klar definiert ist.
 * Um eine Sicht auf die jeweiligen Flächen zu definieren ist daher die Wurfeldrehung festgelegt,
 * durch die die gesuchte Flaeche nach vorne gelangt.
 * Die Notation der Drehung ist dabei bezogen auf
 * http://de.wikipedia.org/wiki/Zauberwürfel#Buchstabennotation</p>
 * 
 * <p>x (rotate): rotate the entire Cube on R<br>
 * y (rotate): rotate the entire Cube on U<br>
 * z (rotate): rotate the entire Cube on F<br>
 *  HINTEN:  y2<br>
 *  RECHTS:  y<br>
 *  LINKS:   y'<br>
 *  OBEN:    x'<br>
 *  UNTEN:   x<br>
 *  </p>
 *  
 *  <p>Zusaetzliche Bewegungsmoeglichkeiten sind nach
 *  http://en.wikipedia.org/wiki/Rubik%27s_Cube#Move_notation
 *  eingebaut (standing, equator, middle).</p>
 * 
 * 
 * @author Matthias Heming
 * @version 1.0 vom 27.1.2014
 */
public class Cube extends Actor
{
    public final static int ROT   =0;
    public final static int BLAU  =1;
    public final static int GELB  =2;
    public final static int WEISS =3;
    public final static int GRUEN =4;
    public final static int ORANGE=5;
    
    public final static int OBEN  =2;
    public final static int UNTEN =3;
    public final static int RECHTS=1;
    public final static int LINKS =4;
    public final static int HINTEN=0;
    public final static int VORNE =5;
    
    
    private int wuerfel[ ][ ][ ];
    // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
    
    private BG world; // Assoziation zur World zur View-Aktualisierung
    
    /**
     * Ein vollstaendig geloester Wuerfel wird erstellt.
    */
    public Cube()
    {
        wuerfel = new int[6][3][3];
        reset();
    }
    
    private void reset() {
        for(int i=0;i<3;i++) for(int j=0;j<3;j++) wuerfel[OBEN][i][j]   = GELB;
        for(int i=0;i<3;i++) for(int j=0;j<3;j++) wuerfel[UNTEN][i][j]  = WEISS;
        for(int i=0;i<3;i++) for(int j=0;j<3;j++) wuerfel[RECHTS][i][j] = BLAU;
        for(int i=0;i<3;i++) for(int j=0;j<3;j++) wuerfel[LINKS][i][j]  = GRUEN;
        for(int i=0;i<3;i++) for(int j=0;j<3;j++) wuerfel[HINTEN][i][j] = ROT;
        for(int i=0;i<3;i++) for(int j=0;j<3;j++) wuerfel[VORNE][i][j]  = ORANGE;
    }
    
    /**
     * Herstellung einer Verbindung zu einem BG-Objekt. Ein solches Objekt
     * kann den Wuerfel visuell darstellen und Drehungen per Tastenkuerzel ausloesen.
     * Falls eine Verbindung besteht, wird jede Aenderung des Wuerfelobjektes per
     * Aufruf der Methode "updateView" an das verbundene BG Objekts mitgeteilt.
     */
    public void connectToWorld(BG pworld) {
        world = pworld;
    }
    
    /**
     * Ein Aufruf dieser Methode gibt an ein eventuell verbundenes BG-Objekt den Hinweis
     * auf eine evtl. Zustandsaenderung des Wuerfels weiter.
     */
    public void updateView() {
        if (world!=null) {
            world.updateView();
        }
    }
    
    /**
     * Diese Methode gibt die aktuelle Farbe einer Wuerfelteilflaeche zurueck.
     * Beachte Hinweis zur Sichtweise auf die unterschiedlichen Wuerfelflaechen!!
     * 
     * @param flaechencode Auf welcher Seite des Wuerfels geschaut werden soll (OBEN, UNTEN, ...)
     * @param x Zeile, nummeriert von 0 bis 2 und von oben nach unten
     *        (auf einer Fläche die Ecke rechts oben haette x=2 und y=0)
     *        (auf einer Fläche die Ecke links unten haette x=0 und y=2)
     * @param y Spalte, nummeriert von 0 bis 2 und von links nach rechts
     *        (auf einer Fläche die Ecke rechts oben haette x=2 und y=0)
     *        (auf einer Fläche die Ecke links unten haette x=0 und y=2)
     */
    public int getColor(int flaechencode, int x, int y) {
        return wuerfel[flaechencode][x][y];
    }
    
    /**
     * Diese Methode gibt die aktuelle Farbe einer Wuerfelteilflaeche zurueck.
     * Beachte Hinweis zur Sichtweise auf die unterschiedlichen Wuerfelflaechen!!
     * 
     * @param cp Ein CubePosition-Objekt, in dem Flaechencode, Zeile und Spalte bereits enthalten ist.
     */
    public int getColor(CubePosition cp) {
        return wuerfel[cp.face][cp.zeile][cp.spalte];
    }
    
    public void right() {
    // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
        int tmp1 = wuerfel[OBEN][0][2];
        int tmp2 = wuerfel[OBEN][1][2];
        int tmp3 = wuerfel[OBEN][2][2];
        wuerfel[OBEN][0][2] = wuerfel[VORNE][0][2];
        wuerfel[OBEN][1][2] = wuerfel[VORNE][1][2];
        wuerfel[OBEN][2][2] = wuerfel[VORNE][2][2];
        wuerfel[VORNE][0][2] = wuerfel[UNTEN][0][2];
        wuerfel[VORNE][1][2] = wuerfel[UNTEN][1][2];
        wuerfel[VORNE][2][2] = wuerfel[UNTEN][2][2];
        wuerfel[UNTEN][0][2] = wuerfel[HINTEN][2][0];
        wuerfel[UNTEN][1][2] = wuerfel[HINTEN][1][0];
        wuerfel[UNTEN][2][2] = wuerfel[HINTEN][0][0];
        wuerfel[HINTEN][0][0] = tmp3;
        wuerfel[HINTEN][1][0] = tmp2;
        wuerfel[HINTEN][2][0] = tmp1;

        int backupkante = wuerfel[RECHTS][0][1];
        wuerfel[RECHTS][0][1] = wuerfel[RECHTS][1][0];
        wuerfel[RECHTS][1][0] = wuerfel[RECHTS][2][1];
        wuerfel[RECHTS][2][1] = wuerfel[RECHTS][1][2];
        wuerfel[RECHTS][1][2] = backupkante;
        
        int backupecke = wuerfel[RECHTS][0][0];
        wuerfel[RECHTS][0][0] = wuerfel[RECHTS][2][0];
        wuerfel[RECHTS][2][0] = wuerfel[RECHTS][2][2];
        wuerfel[RECHTS][2][2] = wuerfel[RECHTS][0][2];
        wuerfel[RECHTS][0][2] = backupecke;
        
        updateView();
    }
    public void rightInverse() {right();right();right();}
    public void right2() {right();right();}
    public void left() {
    // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
        int tmp1 = wuerfel[OBEN][0][0];
        int tmp2 = wuerfel[OBEN][1][0];
        int tmp3 = wuerfel[OBEN][2][0];
        wuerfel[OBEN][0][0] = wuerfel[HINTEN][2][2];
        wuerfel[OBEN][1][0] = wuerfel[HINTEN][1][2];
        wuerfel[OBEN][2][0] = wuerfel[HINTEN][0][2];
        wuerfel[HINTEN][0][2] = wuerfel[UNTEN][2][0];
        wuerfel[HINTEN][1][2] = wuerfel[UNTEN][1][0];
        wuerfel[HINTEN][2][2] = wuerfel[UNTEN][0][0];
        wuerfel[UNTEN][0][0] = wuerfel[VORNE][0][0];
        wuerfel[UNTEN][1][0] = wuerfel[VORNE][1][0];
        wuerfel[UNTEN][2][0] = wuerfel[VORNE][2][0];
        wuerfel[VORNE][0][0] = tmp1;
        wuerfel[VORNE][1][0] = tmp2;
        wuerfel[VORNE][2][0] = tmp3;

        int backupkante = wuerfel[LINKS][0][1];
        wuerfel[LINKS][0][1] = wuerfel[LINKS][1][0];
        wuerfel[LINKS][1][0] = wuerfel[LINKS][2][1];
        wuerfel[LINKS][2][1] = wuerfel[LINKS][1][2];
        wuerfel[LINKS][1][2] = backupkante;
        
        int backupecke = wuerfel[LINKS][0][0];
        wuerfel[LINKS][0][0] = wuerfel[LINKS][2][0];
        wuerfel[LINKS][2][0] = wuerfel[LINKS][2][2];
        wuerfel[LINKS][2][2] = wuerfel[LINKS][0][2];
        wuerfel[LINKS][0][2] = backupecke;
        
        updateView();
    }
    public void leftInverse() {left();left();left();}
    public void left2() {left();left();}
    
    public void up() {
        // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
        int tmp1 = wuerfel[VORNE][0][0];
        int tmp2 = wuerfel[VORNE][0][1];
        int tmp3 = wuerfel[VORNE][0][2];
        wuerfel[VORNE][0][0] = wuerfel[RECHTS][0][0];
        wuerfel[VORNE][0][1] = wuerfel[RECHTS][0][1];
        wuerfel[VORNE][0][2] = wuerfel[RECHTS][0][2];
        wuerfel[RECHTS][0][0] = wuerfel[HINTEN][0][0];
        wuerfel[RECHTS][0][1] = wuerfel[HINTEN][0][1];
        wuerfel[RECHTS][0][2] = wuerfel[HINTEN][0][2];
        wuerfel[HINTEN][0][0] = wuerfel[LINKS][0][0];
        wuerfel[HINTEN][0][1] = wuerfel[LINKS][0][1];
        wuerfel[HINTEN][0][2] = wuerfel[LINKS][0][2];
        wuerfel[LINKS][0][0] = tmp1;
        wuerfel[LINKS][0][1] = tmp2;
        wuerfel[LINKS][0][2] = tmp3;
        
        int backupkante = wuerfel[OBEN][0][1];
        wuerfel[OBEN][0][1] = wuerfel[OBEN][1][0];
        wuerfel[OBEN][1][0] = wuerfel[OBEN][2][1];
        wuerfel[OBEN][2][1] = wuerfel[OBEN][1][2];
        wuerfel[OBEN][1][2] = backupkante;
        
        int backupecke = wuerfel[OBEN][0][0];
        wuerfel[OBEN][0][0] = wuerfel[OBEN][2][0];
        wuerfel[OBEN][2][0] = wuerfel[OBEN][2][2];
        wuerfel[OBEN][2][2] = wuerfel[OBEN][0][2];
        wuerfel[OBEN][0][2] = backupecke;
        
        updateView();
    }
    public void upInverse() {up();up();up();}
    public void up2() {up();up();}
    
    public void down() {
        // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
        int tmp1 = wuerfel[VORNE][2][0];
        int tmp2 = wuerfel[VORNE][2][1];
        int tmp3 = wuerfel[VORNE][2][2];
        wuerfel[VORNE][2][0] = wuerfel[LINKS][2][0];
        wuerfel[VORNE][2][1] = wuerfel[LINKS][2][1];
        wuerfel[VORNE][2][2] = wuerfel[LINKS][2][2];
        wuerfel[LINKS][2][0] = wuerfel[HINTEN][2][0];
        wuerfel[LINKS][2][1] = wuerfel[HINTEN][2][1];
        wuerfel[LINKS][2][2] = wuerfel[HINTEN][2][2];
        wuerfel[HINTEN][2][0] = wuerfel[RECHTS][2][0];
        wuerfel[HINTEN][2][1] = wuerfel[RECHTS][2][1];
        wuerfel[HINTEN][2][2] = wuerfel[RECHTS][2][2];
        wuerfel[RECHTS][2][0] = tmp1;
        wuerfel[RECHTS][2][1] = tmp2;
        wuerfel[RECHTS][2][2] = tmp3;
        
        int backupkante = wuerfel[UNTEN][0][1];
        wuerfel[UNTEN][0][1] = wuerfel[UNTEN][1][0];
        wuerfel[UNTEN][1][0] = wuerfel[UNTEN][2][1];
        wuerfel[UNTEN][2][1] = wuerfel[UNTEN][1][2];
        wuerfel[UNTEN][1][2] = backupkante;
        
        int backupecke = wuerfel[UNTEN][0][0];
        wuerfel[UNTEN][0][0] = wuerfel[UNTEN][2][0];
        wuerfel[UNTEN][2][0] = wuerfel[UNTEN][2][2];
        wuerfel[UNTEN][2][2] = wuerfel[UNTEN][0][2];
        wuerfel[UNTEN][0][2] = backupecke;
        
        updateView();
    }
    public void downInverse() {down();down();down();}
    public void down2() {down();down();}
    
    public void back() {
        // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
        int tmp1 = wuerfel[OBEN][0][0];
        int tmp2 = wuerfel[OBEN][0][1];
        int tmp3 = wuerfel[OBEN][0][2];
        wuerfel[OBEN][0][0] = wuerfel[RECHTS][0][2];
        wuerfel[OBEN][0][1] = wuerfel[RECHTS][1][2];
        wuerfel[OBEN][0][2] = wuerfel[RECHTS][2][2];
        wuerfel[RECHTS][0][2] = wuerfel[UNTEN][2][2];
        wuerfel[RECHTS][1][2] = wuerfel[UNTEN][2][1];
        wuerfel[RECHTS][2][2] = wuerfel[UNTEN][2][0];
        wuerfel[UNTEN][2][0] = wuerfel[LINKS][0][0];
        wuerfel[UNTEN][2][1] = wuerfel[LINKS][1][0];
        wuerfel[UNTEN][2][2] = wuerfel[LINKS][2][0];
        wuerfel[LINKS][0][0] = tmp3;
        wuerfel[LINKS][1][0] = tmp2;
        wuerfel[LINKS][2][0] = tmp1;
        
        int backupkante = wuerfel[HINTEN][0][1];
        wuerfel[HINTEN][0][1] = wuerfel[HINTEN][1][0];
        wuerfel[HINTEN][1][0] = wuerfel[HINTEN][2][1];
        wuerfel[HINTEN][2][1] = wuerfel[HINTEN][1][2];
        wuerfel[HINTEN][1][2] = backupkante;
        
        int backupecke = wuerfel[HINTEN][0][0];
        wuerfel[HINTEN][0][0] = wuerfel[HINTEN][2][0];
        wuerfel[HINTEN][2][0] = wuerfel[HINTEN][2][2];
        wuerfel[HINTEN][2][2] = wuerfel[HINTEN][0][2];
        wuerfel[HINTEN][0][2] = backupecke;
        
        updateView();
    }
    public void backInverse() {back();back();back();}
    public void back2() {back();back();}
    
    public void front() {
        // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
        int tmp1 = wuerfel[OBEN][2][0];
        int tmp2 = wuerfel[OBEN][2][1];
        int tmp3 = wuerfel[OBEN][2][2];
        wuerfel[OBEN][2][0] = wuerfel[LINKS][2][2];
        wuerfel[OBEN][2][1] = wuerfel[LINKS][1][2];
        wuerfel[OBEN][2][2] = wuerfel[LINKS][0][2];
        wuerfel[LINKS][0][2] = wuerfel[UNTEN][0][0];
        wuerfel[LINKS][1][2] = wuerfel[UNTEN][0][1];
        wuerfel[LINKS][2][2] = wuerfel[UNTEN][0][2];
        wuerfel[UNTEN][0][0] = wuerfel[RECHTS][2][0];
        wuerfel[UNTEN][0][1] = wuerfel[RECHTS][1][0];
        wuerfel[UNTEN][0][2] = wuerfel[RECHTS][0][0];
        wuerfel[RECHTS][0][0] = tmp1;
        wuerfel[RECHTS][1][0] = tmp2;
        wuerfel[RECHTS][2][0] = tmp3;
        
        int backupkante = wuerfel[VORNE][0][1];
        wuerfel[VORNE][0][1] = wuerfel[VORNE][1][0];
        wuerfel[VORNE][1][0] = wuerfel[VORNE][2][1];
        wuerfel[VORNE][2][1] = wuerfel[VORNE][1][2];
        wuerfel[VORNE][1][2] = backupkante;
        
        int backupecke = wuerfel[VORNE][0][0];
        wuerfel[VORNE][0][0] = wuerfel[VORNE][2][0];
        wuerfel[VORNE][2][0] = wuerfel[VORNE][2][2];
        wuerfel[VORNE][2][2] = wuerfel[VORNE][0][2];
        wuerfel[VORNE][0][2] = backupecke;
        
        updateView();
    }
    public void frontInverse() {front();front();front();}
    public void front2() {front();front();}
    
    public void middle() {
    //TOP->DOWN
    // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
        int tmp1 = wuerfel[OBEN][0][1];
        int tmp2 = wuerfel[OBEN][1][1];
        int tmp3 = wuerfel[OBEN][2][1];
        wuerfel[OBEN][0][1] = wuerfel[HINTEN][2][1];
        wuerfel[OBEN][1][1] = wuerfel[HINTEN][1][1];
        wuerfel[OBEN][2][1] = wuerfel[HINTEN][0][1];
        wuerfel[HINTEN][0][1] = wuerfel[UNTEN][2][1];
        wuerfel[HINTEN][1][1] = wuerfel[UNTEN][1][1];
        wuerfel[HINTEN][2][1] = wuerfel[UNTEN][0][1];
        wuerfel[UNTEN][0][1] = wuerfel[VORNE][0][1];
        wuerfel[UNTEN][1][1] = wuerfel[VORNE][1][1];
        wuerfel[UNTEN][2][1] = wuerfel[VORNE][2][1];
        wuerfel[VORNE][0][1] = tmp1;
        wuerfel[VORNE][1][1] = tmp2;
        wuerfel[VORNE][2][1] = tmp3;
        
        updateView();
    }
    public void middleInverse() {middle();middle();middle();}
    public void middle2() {middle();middle();}
    

    public void equator() {
    //LEFT->RIGHT
    // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
        int tmp1 = wuerfel[VORNE][1][0];
        int tmp2 = wuerfel[VORNE][1][1];
        int tmp3 = wuerfel[VORNE][1][2];
        wuerfel[VORNE][1][0] = wuerfel[LINKS][1][0];
        wuerfel[VORNE][1][1] = wuerfel[LINKS][1][1];
        wuerfel[VORNE][1][2] = wuerfel[LINKS][1][2];
        wuerfel[LINKS][1][0] = wuerfel[HINTEN][1][0];
        wuerfel[LINKS][1][1] = wuerfel[HINTEN][1][1];
        wuerfel[LINKS][1][2] = wuerfel[HINTEN][1][2];
        wuerfel[HINTEN][1][0] = wuerfel[RECHTS][1][0];
        wuerfel[HINTEN][1][1] = wuerfel[RECHTS][1][1];
        wuerfel[HINTEN][1][2] = wuerfel[RECHTS][1][2];
        wuerfel[RECHTS][1][0] = tmp1;
        wuerfel[RECHTS][1][1] = tmp2;
        wuerfel[RECHTS][1][2] = tmp3;
        
        updateView();
    }
    public void equatorInverse() {equator();equator();equator();}
    public void equator2() {equator();equator();}
    
    public void standing() {
    //RIGHT SIDE: TOP->DOWN
    // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
        int tmp1 = wuerfel[OBEN][1][0];
        int tmp2 = wuerfel[OBEN][1][1];
        int tmp3 = wuerfel[OBEN][1][2];
        wuerfel[OBEN][1][0] = wuerfel[LINKS][2][1];
        wuerfel[OBEN][1][1] = wuerfel[LINKS][1][1];
        wuerfel[OBEN][1][2] = wuerfel[LINKS][0][1];
        wuerfel[LINKS][0][1] = wuerfel[UNTEN][1][0];
        wuerfel[LINKS][1][1] = wuerfel[UNTEN][1][1];
        wuerfel[LINKS][2][1] = wuerfel[UNTEN][1][2];
        wuerfel[UNTEN][1][0] = wuerfel[RECHTS][2][1];
        wuerfel[UNTEN][1][1] = wuerfel[RECHTS][1][1];
        wuerfel[UNTEN][1][2] = wuerfel[RECHTS][0][1];
        wuerfel[RECHTS][0][1] = tmp1;
        wuerfel[RECHTS][1][1] = tmp2;
        wuerfel[RECHTS][2][1] = tmp3;
        
        updateView();
    }
    public void standingInverse() {standing();standing();standing();}
    public void standing2() {standing();standing();}
    
    public void smallRight() {right();middleInverse();}
    public void smallRightInverse() {rightInverse();middle();}
    public void smallRight2() {right2();middle2();}
    public void smallLeft() {left();middle();}
    public void smallLeftInverse() {leftInverse();middleInverse();}
    public void smallLeft2() {left2();middle2();}
    public void smallDown() {down();equator();}
    public void smallDownInverse() {downInverse();equatorInverse();}
    public void smallDown2() {down2();equator2();}
    public void smallUp() {up();equatorInverse();}
    public void smallUpInverse() {upInverse();equator();}
    public void smallUp2() {up2();equator();}
    public void smallBack() {back();standingInverse();}
    public void smallBackInverse() {backInverse();standing();}
    public void smallBack2() {back2();standing2();}
    public void smallFront() {front();standing();}
    public void smallFrontInverse() {frontInverse();standingInverse();}
    public void smallFront2() {front2();standing2();}

    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F

    public void xCubeRotate() {smallRight();leftInverse();}
    public void yCubeRotate() {smallUp();downInverse();}
    public void zCubeRotate() {smallFront();backInverse();}
    
    /**
     * Diese Methode sucht nach einer Kante, die durch die angegebenen Farben definiert ist.
     * Die erste Farbe definiert die Teilflaeche des Wuerfels, die als Positionsobjekt zurueckgegeben wird.
     * Wird z.B. nach ROT/BLAU gesucht, so wird die Kante gesucht, jedoch nur die Position der ROTEN Teilflaeche zurueckgegeben.
     * @param color1 Die erste Farbe, deren Position zurueckgegeben wird.
     * @param color2 Die zweite Farbe, die zur Unterscheidung der vier unterschiedlichen Kanten zusaetzlich angegeben werden muss.
     * @return Ein CubePosition-Objekt fuer die Position des Kantenstuecks mit der Farbe color1.
     */
    public CubePosition searchForEdge(int color1, int color2) {
    // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
        if (wuerfel[VORNE][0][1]==color1 && wuerfel[OBEN][2][1]==color2)    return new CubePosition(VORNE,0,1);
        if (wuerfel[VORNE][1][2]==color1 && wuerfel[RECHTS][1][0]==color2)  return new CubePosition(VORNE,1,2);
        if (wuerfel[VORNE][1][0]==color1 && wuerfel[LINKS][1][2]==color2)   return new CubePosition(VORNE,1,0);
        if (wuerfel[VORNE][2][1]==color1 && wuerfel[UNTEN][0][1]==color2)   return new CubePosition(VORNE,2,1);
        if (wuerfel[VORNE][0][1]==color2 && wuerfel[OBEN][2][1]==color1)    return new CubePosition(OBEN,2,1);
        if (wuerfel[VORNE][1][2]==color2 && wuerfel[RECHTS][1][0]==color1)  return new CubePosition(RECHTS,1,0);
        if (wuerfel[VORNE][1][0]==color2 && wuerfel[LINKS][1][2]==color1)   return new CubePosition(LINKS,1,2);
        if (wuerfel[VORNE][2][1]==color2 && wuerfel[UNTEN][0][1]==color1)   return new CubePosition(UNTEN,0,1);
        
        if (wuerfel[OBEN][1][0]==color1 && wuerfel[LINKS][0][1]==color2)    return new CubePosition(OBEN,1,0);
        if (wuerfel[OBEN][1][2]==color1 && wuerfel[RECHTS][0][1]==color2)   return new CubePosition(OBEN,1,2);
        if (wuerfel[UNTEN][1][0]==color1 && wuerfel[LINKS][2][1]==color2)   return new CubePosition(UNTEN,1,0);
        if (wuerfel[UNTEN][1][2]==color1 && wuerfel[RECHTS][2][1]==color2)  return new CubePosition(UNTEN,1,2);
        if (wuerfel[OBEN][1][0]==color2 && wuerfel[LINKS][0][1]==color1)    return new CubePosition(LINKS,0,1);
        if (wuerfel[OBEN][1][2]==color2 && wuerfel[RECHTS][0][1]==color1)   return new CubePosition(RECHTS,0,1);
        if (wuerfel[UNTEN][1][0]==color2 && wuerfel[LINKS][2][1]==color1)   return new CubePosition(LINKS,2,1);
        if (wuerfel[UNTEN][1][2]==color2 && wuerfel[RECHTS][2][1]==color1)  return new CubePosition(RECHTS,2,1);
        
        if (wuerfel[HINTEN][0][1]==color1 && wuerfel[OBEN][0][1]==color2)    return new CubePosition(HINTEN,0,1);
        if (wuerfel[HINTEN][1][2]==color1 && wuerfel[LINKS][1][0]==color2)   return new CubePosition(HINTEN,1,2);
        if (wuerfel[HINTEN][1][0]==color1 && wuerfel[RECHTS][1][2]==color2)  return new CubePosition(HINTEN,1,0);
        if (wuerfel[HINTEN][2][1]==color1 && wuerfel[UNTEN][2][1]==color2)   return new CubePosition(HINTEN,2,1);
        if (wuerfel[HINTEN][0][1]==color2 && wuerfel[OBEN][0][1]==color1)    return new CubePosition(OBEN,0,1);
        if (wuerfel[HINTEN][1][2]==color2 && wuerfel[LINKS][1][0]==color1)   return new CubePosition(LINKS,1,0);
        if (wuerfel[HINTEN][1][0]==color2 && wuerfel[RECHTS][1][2]==color1)  return new CubePosition(RECHTS,1,2);
        if (wuerfel[HINTEN][2][1]==color2 && wuerfel[UNTEN][2][1]==color1)   return new CubePosition(UNTEN,2,1);

        else return null;
    }
    
    /**
     * Diese Methode sucht nach einer Ecke, die durch die angegebenen Farben definiert ist.
     * Die erste Farbe definiert die Teilflaeche des Wuerfels, die als Positionsobjekt zurueckgegeben wird.
     * Die dritte Farbe der Ecke wird weggelassen, da die Vorgabe gemacht wird, dass die Farben jeweils im Uhrzeigersinn gesehen werden und
     * damit die dritte Farbe bereits feststeht.
     * Die Ecke ROT/BLAU entspricht damit der Ecke ROT/BLAU/GELB, wobei die Ecke BLAU/ROT der Ecke BLAU/ROT/WEISS entspricht.
     * 
     * Wird z.B. nach ROT/WEISS/BLAU gesucht, so wird die Ecke gesucht, jedoch nur die Position der ROTEN Teilflaeche zurueckgegeben.
     * @param color1 Die erste Farbe, deren Position zurueckgegeben wird.
     * @param color2 Die zweite Farbe, die zur Unterscheidung der vier unterschiedlichen Kanten zusaetzlich angegeben werden muss.
     * @return Ein CubePosition-Objekt fuer die Position des Kantenstuecks mit der Farbe color1.
     */
    public CubePosition searchForCorner(int color1, int color2) {
        // dim 1: welche Flaeche
    // dim 2/3: x-y-Koordinate nach der folgenden Drehung
    // x (rotate): rotate the entire Cube on R
    // y (rotate): rotate the entire Cube on U
    // z (rotate): rotate the entire Cube on F
    //     HINTEN:  y2
    //     RECHTS:  y
    //     LINKS:   y'
    //     OBEN:    x'
    //     UNTEN:   x
    
    // corner colors color1 and color 2 clockwise
        if (wuerfel[OBEN][2][0]==color1 && wuerfel[VORNE][0][0]==color2)  return new CubePosition(OBEN,2,0);
         if (wuerfel[VORNE][0][0]==color1 && wuerfel[LINKS][0][2]==color2)  return new CubePosition(VORNE,0,0);
         if (wuerfel[LINKS][0][2]==color1 && wuerfel[OBEN][2][0]==color2)  return new CubePosition(LINKS,0,2);
        if (wuerfel[OBEN][2][2]==color1 && wuerfel[RECHTS][0][0]==color2)  return new CubePosition(OBEN,2,2);
         if (wuerfel[RECHTS][0][0]==color1 && wuerfel[VORNE][0][2]==color2)  return new CubePosition(RECHTS,0,0);
         if (wuerfel[VORNE][0][2]==color1 && wuerfel[OBEN][2][2]==color2)  return new CubePosition(VORNE,0,2); 
        if (wuerfel[OBEN][0][0]==color1 && wuerfel[LINKS][0][0]==color2)  return new CubePosition(OBEN,0,0);
         if (wuerfel[LINKS][0][0]==color1 && wuerfel[HINTEN][0][2]==color2)  return new CubePosition(LINKS,0,0);
         if (wuerfel[HINTEN][0][2]==color1 && wuerfel[OBEN][0][0]==color2)  return new CubePosition(HINTEN,0,2);
        if (wuerfel[OBEN][0][2]==color1 && wuerfel[HINTEN][0][0]==color2)  return new CubePosition(OBEN,2,2);
         if (wuerfel[HINTEN][0][0]==color1 && wuerfel[RECHTS][0][2]==color2)  return new CubePosition(HINTEN,0,0);
         if (wuerfel[RECHTS][0][2]==color1 && wuerfel[OBEN][0][2]==color2)  return new CubePosition(RECHTS,0,2);
         
        if (wuerfel[UNTEN][0][2]==color1 && wuerfel[VORNE][2][2]==color2)  return new CubePosition(UNTEN,0,2);
         if (wuerfel[VORNE][2][2]==color1 && wuerfel[RECHTS][2][0]==color2)  return new CubePosition(VORNE,2,2);
         if (wuerfel[RECHTS][2][0]==color1 && wuerfel[OBEN][2][0]==color2)  return new CubePosition(RECHTS,2,0);
        if (wuerfel[UNTEN][0][0]==color1 && wuerfel[LINKS][2][2]==color2)  return new CubePosition(UNTEN,0,0);
         if (wuerfel[LINKS][2][2]==color1 && wuerfel[VORNE][2][0]==color2)  return new CubePosition(LINKS,2,2);
         if (wuerfel[VORNE][2][0]==color1 && wuerfel[UNTEN][0][0]==color2)  return new CubePosition(VORNE,2,0); 
        if (wuerfel[UNTEN][2][0]==color1 && wuerfel[HINTEN][2][2]==color2)  return new CubePosition(UNTEN,2,0);
         if (wuerfel[HINTEN][2][2]==color1 && wuerfel[LINKS][2][0]==color2)  return new CubePosition(HINTEN,2,2);
         if (wuerfel[LINKS][2][0]==color1 && wuerfel[UNTEN][2][0]==color2)  return new CubePosition(LINKS,2,0);
        if (wuerfel[UNTEN][2][2]==color1 && wuerfel[RECHTS][2][2]==color2)  return new CubePosition(UNTEN,2,2);
         if (wuerfel[RECHTS][2][2]==color1 && wuerfel[HINTEN][2][0]==color2)  return new CubePosition(RECHTS,2,2);
         if (wuerfel[HINTEN][2][0]==color1 && wuerfel[UNTEN][2][2]==color2)  return new CubePosition(HINTEN,2,0);
        
        return null;
    }
    
    /**
     * Verdreht den Wuerfel nach folgendem Algorithmus RLURD2LFD2LB.
     */
    public void scramble() {
        right();
        left();
        up();
        right();
        down2();
        left();
        front();
        down2();
        left();
        back();
    }
    
    /**
     * Diese Methode uebersetzt die als int uebergebene Farbe in ein String-Objekt (u.a. zu Debug-Zwecken verwendet).
     */
    public String color2String(int color) {
        if (color==ROT) return "ROT";
        if (color==GRUEN) return "GRUEN";
        if (color==BLAU) return "BLAU";
        if (color==GELB) return "GELB";
        if (color==WEISS) return "WEISS";
        if (color==ORANGE) return "ORANGE";
        return "";
    }
    
    /**
     * Methode, die einen beliebig verdrehten Wuerfel in seine Ausgangsposition zurueckdrehen soll (TODO!!).
     */
    public void loeseWuerfel() {
        //CubePosition cp;
        //cp = searchForEdge(WEISS,GRUEN);
        //if (cp.face==OBEN && cp.zeile==0 && cp.spalte == 1) {back2();down2();front2();}
        //if (cp.face==OBEN && cp.zeile==0 && cp.spalte == 1) {back2();down2();front2();}
    }
    
}